

struct vertOUT
{
	float4 hPos :POSITION;
	float4 hPosition :TEXCOORD4;
	float3 wPos	:TEXCOORD1;
	float2 texCoord :TEXCOORD0;
	float3 mNormal :TEXCOORD2;
};

vertOUT PhongVS(float4 position : POSITION,
		float3 normal   : NORMAL,
		half3 tangent   : TEXCOORD1, 
		float2 texCoord : TEXCOORD0,			
		uniform float4x4 worldViewProj,
		uniform float4x4 world)
{
  vertOUT OUT;
  OUT.hPos = OUT.hPosition = mul(worldViewProj, position);
  OUT.wPos = mul(world, position).xyz;  
  OUT.mNormal = mul(world, normal);
  //OUT.mNormal = normal;
  OUT.texCoord = texCoord;
  return OUT;
}



float4 PhongPS( vertOUT IN,
           uniform float3 cameraPos,
	   uniform float3 lightpos1,
	   uniform float3 lightpos2,
	   uniform float3 lightpos3,
	   uniform float3 lightpos4,
	   uniform float3 lightpos5,
	   uniform float3 lightpos6,
	   uniform float3 lightpos7,
	   uniform float3 lightpos8,
	   uniform float4 lightcol1,
	   uniform float4 lightcol2,
	   uniform float4 lightcol3,
	   uniform float4 lightcol4,
	   uniform float4 lightcol5,
	   uniform float4 lightcol6,
	   uniform float4 lightcol7,
	   uniform float4 lightcol8,
	   uniform float4 ambientLight,
	   uniform float shininess,
	   uniform float4 specular,
	   uniform float4 diffuse,
	   uniform float4 ambient           

            ) : COLOR0
{
   
	
    float3 N = IN.mNormal;
    N = normalize( N ); 
    float3 pos = IN.wPos;
    
    float M = 1;
   
    float3 lightpositions[8];
    float4 lightcolors[8];

    lightpositions[0] = lightpos1;
    lightpositions[1] = lightpos2;
    lightpositions[2] = lightpos3;
    lightpositions[3] = lightpos4;
    lightpositions[4] = lightpos5;
    lightpositions[5] = lightpos6;
    lightpositions[6] = lightpos7;
    lightpositions[7] = lightpos8;

    lightcolors[0] = lightcol1;
    lightcolors[1] = lightcol2;
    lightcolors[2] = lightcol3;
    lightcolors[3] = lightcol4;
    lightcolors[4] = lightcol5;
    lightcolors[5] = lightcol6;
    lightcolors[6] = lightcol7;
    lightcolors[7] = lightcol8;

    float4 I = ambientLight * ambient;
	
    for(int i=0; i< M; i++)
    {
	float3 V = normalize(cameraPos - pos);
	float3 L = lightpositions[i] - pos;
	float d = length(L);
	L = normalize(L);
	float3 H = normalize(L + V);
	float lightscale = 1.0;
	float4 spec = specular * lightcolors[i] * pow(dot(H, N), shininess) * lightscale;
	float4 diff = diffuse * lightcolors[i] * abs(dot(L, N)) * lightscale;
	float range = 4.0;
	I += 1.0 / d * (spec + diff);	
    }

//return 1;
    return I;

}


float4 PhongPlaneReflectPS( vertOUT IN,
           uniform float3 cameraPos,
	   uniform float3 lightpos1,
	   uniform float3 lightpos2,
	   uniform float3 lightpos3,
	   uniform float3 lightpos4,
	   uniform float3 lightpos5,
	   uniform float3 lightpos6,
	   uniform float3 lightpos7,
	   uniform float3 lightpos8,
	   uniform float4 lightcol1,
	   uniform float4 lightcol2,
	   uniform float4 lightcol3,
	   uniform float4 lightcol4,
	   uniform float4 lightcol5,
	   uniform float4 lightcol6,
	   uniform float4 lightcol7,
	   uniform float4 lightcol8,
	   uniform float4 ambientLight,
	   uniform float shininess,
	   uniform float4 specular,
	   uniform float4 diffuse,
	   uniform float4 ambient,
	   uniform sampler2D reflectionTex,
	   uniform float reflectionAmount         

            ) : COLOR0
{
   
	
    float3 N = IN.mNormal;
    N = normalize( N ); 
    float3 pos = IN.wPos;
    
    float M = 4;
   
    float3 lightpositions[8];
    float4 lightcolors[8];

    lightpositions[0] = lightpos1;
    lightpositions[1] = lightpos2;
    lightpositions[2] = lightpos3;
    lightpositions[3] = lightpos4;
    lightpositions[4] = lightpos5;
    lightpositions[5] = lightpos6;
    lightpositions[6] = lightpos7;
    lightpositions[7] = lightpos8;

    lightcolors[0] = lightcol1;
    lightcolors[1] = lightcol2;
    lightcolors[2] = lightcol3;
    lightcolors[3] = lightcol4;
    lightcolors[4] = lightcol5;
    lightcolors[5] = lightcol6;
    lightcolors[6] = lightcol7;
    lightcolors[7] = lightcol8;

    float4 I = ambientLight * ambient;
	
    for(int i=0; i< M; i++)
    {
	float3 V = normalize(cameraPos - pos);
	float3 L = lightpositions[i] - pos;
	float d = length(L);
	L = normalize(L);
	float3 H = normalize(L + V);
	float lightscale = 3;
	float4 spec = specular * lightcolors[i] * pow(dot(H, N), shininess) * lightscale;
	float4 diff = diffuse * lightcolors[i] * abs(dot(L, N)) * lightscale;
	float range = 4.0;
	I += 1.0 / d * (spec + diff);	
    }
    
    float2 uv = (IN.hPosition.xy/IN.hPosition.w + 1.0) * 0.5;
    uv.y = 1.0 - uv.y;
    float4 reflection = tex2D(reflectionTex, uv);
    I *= reflectionAmount * reflection + (1 - reflectionAmount);

//I = reflection;
//return 1;
    return I;

}
